import fsExtra from 'fs-extra';
import createDebugger from 'debug';
export async function generatePackageClass(param) {
    const debug = createDebugger('generatePackageClass');
    debug({ param });
    const { deepJsonFilePath, outputFilePath, packageName } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    const ownedLinks = deepJson.data.filter((link) => typeof link.id === 'string');
    debug({ ownedLinks });
    let classDefinition = `
  /**
   * Represents a deep package
   * 
   * @remarks
   * Contains name of the package and all the links as the objects with id method which returns the id of the link.
   * 
   * @example
\`\`\`ts
const package = nwe Package({deep});
const {name: packageName} = package;
const ${ownedLinks[0].id}LinkId = await package.${ownedLinks[0].id}.id();
\`\`\`
   */
  import { DeepClient } from '@deep-foundation/deeplinks/imports/client';

  export class Package {
    private deep: DeepClient;
    /**
     * Name of the package
     */
    public name: string = '${packageName}';

    constructor(deep: DeepClient) {
      this.deep = deep;
    }
  
    private createEntity(...names: string[]) {
      return {
        id: async () => {
          return await this.id(this.name, ...names);
        },
      };
    }
  
    async id(...names: string[]) {
      return await this.deep.id(this.name, ...names);
    }
${ownedLinks
        .map(({ id }) => `
    /**
     * @example
\`\`\`ts
const package = new Package({deep});
const ${id}LinkId = await package.${id}.id();
\`\`\`
     */
    public ${id} = this.createEntity("${id}");`)
        .join('')}

  }
`;
    debug({ classDefinition });
    await fsExtra.writeFile(outputFilePath, classDefinition);
}
